<?php

namespace Drupal\ai_provider_docker\Client;

use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Embeddings\EmbeddingsInput;
use OpenAI\Client;

/**
 * Interface for the AI client.
 */
class OpenAiClient implements AiClientInterface {

  /**
   * The OpenAI client instance.
   *
   * @var \OpenAI\Client
   */
  protected Client $client;

  public function __construct(Client $client) {
    $this->client = $client;
  }

  /**
   * {@inheritdoc}
   */
  public function getModels(): array {
    $models = [];
    $list = $this->client->models()->list()->toArray();
    foreach ($list['data'] as $model) {
      $models[$model['id']] = $model['id'];
    }
    return $models;
  }

  /**
   * {@inheritdoc}
   */
  public function buildChatPayload($input, string $model_id, array $tags, mixed $chat_system_role, array $configuration): array {
    $chat_input = $input instanceof ChatInput ? $this->transformChatInput($input, $chat_system_role) : $input;

    return [
      'model' => $model_id,
      'messages' => $chat_input,
    ] + $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function executeChatRequest(array $payload, bool $streamed = FALSE): array {
    if ($streamed) {
      return [$this->client->chat()->createStreamed($payload)];
    }

    $response = $this->client->chat()->create($payload)->toArray();

    return [
      'role' => $response['choices'][0]['message']['role'],
      'text' => $response['choices'][0]['message']['content'] ?? '',
    ];
  }

  /**
   * Transforms ChatInput into the required format.
   *
   * @param \Drupal\ai\OperationType\Chat\ChatInput $input
   *   The chat input.
   * @param string $chat_system_role
   *   The system role for the chat.
   *
   * @return array
   *   The transformed input.
   */
  protected function transformChatInput(ChatInput $input, string $chat_system_role): array {
    $chat_input = [];
    if ($chat_system_role) {
      $chat_input[] = ['role' => 'system', 'content' => $chat_system_role];
    }
    foreach ($input->getMessages() as $message) {
      $chat_input[] = [
        'role' => $message->getRole(),
        'content' => [['type' => 'text', 'text' => $message->getText()]],
      ];
    }
    return $chat_input;
  }

  /**
   * {@inheritdoc}
   */
  public function buildEmbeddingsPayload(string|EmbeddingsInput $input, string $model_id, array $tags, array $configuration): array {
    $input_text = $input instanceof EmbeddingsInput ? $input->getPrompt() : $input;

    return [
      'model' => $model_id,
      'input' => $input_text,
    ] + $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function executeEmbeddingsRequest(array $payload): array {
    $response = $this->client->embeddings()->create($payload)->toArray();
    return [
      'embedding' => $response['data'][0]['embedding'] ?? [],
      'raw_output' => $response,
    ];

  }

}
