<?php

namespace Drupal\ai_provider_docker\Plugin\AiProvider;

use Drupal\ai_provider_docker\Client\AiClientFactory;
use Drupal\ai_provider_docker\Client\AiClientInterface;
use Drupal\ai\Attribute\AiProvider;
use Drupal\ai\Base\AiProviderClientBase;
use Drupal\ai\OperationType\Chat\ChatInput;
use Drupal\ai\OperationType\Chat\ChatInterface;
use Drupal\ai\OperationType\Chat\ChatMessage;
use Drupal\ai\OperationType\Chat\ChatOutput;
use Drupal\ai\OperationType\Embeddings\EmbeddingsInput;
use Drupal\ai\OperationType\Embeddings\EmbeddingsInterface;
use Drupal\ai\OperationType\Embeddings\EmbeddingsOutput;
use Drupal\ai\Traits\OperationType\ChatTrait;
use Drupal\ai_provider_docker\DockerModelRunnerChatMessageIterator;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\Yaml\Yaml;

/**
 * Defines the Docker Model Runner Provider plugin.
 */
#[AiProvider(
  id: 'docker_model_runner',
  label: new TranslatableMarkup('Docker Model Runner')
)]
class DockerModelRunnerProvider extends AiProviderClientBase implements ChatInterface, EmbeddingsInterface {

  use ChatTrait;

  /**
   * The AI client interface.
   *
   * @var \Drupal\ai_provider_docker\Client\AiClientInterface|null
   */
  protected AiClientInterface|null $client = NULL;

  /**
   * {@inheritdoc}
   */
  public function getConfiguredModels(?string $operation_type = NULL, array $capabilities = []): array {
    $this->loadClient();

    $cache_key = 'docker_model_runner_' . $operation_type . '_' . Crypt::hashBase64(Json::encode($capabilities));
    $cache_data = $this->cacheBackend->get($cache_key);

    if (!empty($cache_data)) {
      return $cache_data->data;
    }

    $models = [];
    $list = $this->client->getModels();
    // Filter models based on the operation type.
    foreach ($list as $model) {
      $isEmbeddingModel = strpos($model, '-embed-') !== FALSE;

      if (($operation_type === 'embeddings' && $isEmbeddingModel) ||
        ($operation_type === 'chat' && !$isEmbeddingModel)) {
        $models[$model] = $model;
      }
    }

    if (!empty($models)) {
      asort($models);
      $this->cacheBackend->set($cache_key, $models);
    }

    return $models;
  }

  /**
   * {@inheritdoc}
   */
  public function isUsable(?string $operation_type = NULL, array $capabilities = []): bool {
    if ($operation_type) {
      return in_array($operation_type, $this->getSupportedOperationTypes());
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedOperationTypes(): array {
    return [
      'chat',
      'embeddings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(): ImmutableConfig {
    return $this->configFactory->get('ai_provider_docker.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function getApiDefinition(): array {
    $definition = Yaml::parseFile(
      $this->moduleHandler->getModule('ai_provider_docker')
        ->getPath() . '/definitions/api_defaults.yml'
    );
    return $definition;
  }

  /**
   * {@inheritdoc}
   */
  public function getModelSettings(string $model_id, array $generalConfig = []): array {
    return $generalConfig;
  }

  /**
   * {@inheritdoc}
   */
  public function setAuthentication(mixed $authentication): void {
    $this->client = NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function chat(array|string|ChatInput $input, string $model_id, array $tags = []): ChatOutput {
    $this->loadClient();

    $payload = $this->client->buildChatPayload($input, $model_id, $tags, $this->chatSystemRole, $this->configuration);

    try {
      if ($this->streamed) {
        $response = $this->client->executeChatRequest($payload, TRUE);
        $message = new DockerModelRunnerChatMessageIterator(reset($response));
      }
      else {
        $response = $this->client->executeChatRequest($payload);
        $message = new ChatMessage($response['role'], $response['text'], []);
      }

    }
    catch (\Exception $e) {
      throw $e;
    }

    return new ChatOutput($message, $response, []);
  }

  /**
   * Returns the AiClient Client.
   *
   * @return \Drupal\ai_provider_docker\Client\AiClientInterface
   *   The AiClient Client.
   */
  public function getClient(): AiClientInterface {
    $this->loadClient();
    return $this->client;
  }

  /**
   * Loads the AiClient Client.
   */
  protected function loadClient(): void {
    if (!$this->client) {
      $this->client = AiClientFactory::createClient($this->httpClient, $this->getConfig());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function embeddings(string|EmbeddingsInput $input, string $model_id, array $tags = []): EmbeddingsOutput {
    $this->loadClient();

    $payload = $this->client->buildEmbeddingsPayload($input, $model_id, $tags, $this->configuration);
    try {
      $response = $this->client->executeEmbeddingsRequest($payload);
    }
    catch (\Exception $e) {
      throw $e;
    }

    return new EmbeddingsOutput($response['embedding'], $response['raw_output'], []);

  }

  /**
   * {@inheritdoc}
   */
  public function maxEmbeddingsInput($model_id = ''): int {
    // @todo this is playing safe. Ideally, we should provide real number per model.
    return 2048;
  }

  /**
   * {@inheritdoc}
   */
  public function embeddingsVectorSize(string $model_id): int {
    return match($model_id) {
      'ai/mxbai-embed-large' => 1024,
      default => 0,
    };

  }

}
